@@ -2,4 +2,4 @@ struct test_redirections {
   ssize_t (*write)(int fd, const void *buf, size_t count);
 };
 
-struct test_redirections *test_redirections();
+struct test_redirections *test_redirections(void);
